@@ -2,6 +2,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
+import copy
 import json
 import shutil
 import subprocess
@@ -10,7 +11,7 @@ from pathlib import Path
 import click
 import pkg_resources
 import yaml
-from inflection import titleize
+from inflection import dasherize, titleize
 from loguru import logger
 from packaging.version import InvalidVersion
 from packaging.version import parse as parse_version
@@ -23,7 +24,12 @@ from reflekt.loader import ReflektLoader
 from reflekt.logger import logger_config
 from reflekt.project import ReflektProject
 from reflekt.segment.plan import SegmentPlan
+from reflekt.segment.schema import (  # TODO: Likely need a handler for these schema if build out Rudderstack & Snowplow  # noqa: E501
+    segment_payload_schema,
+    segment_plan_schema,
+)
 from reflekt.transformer import ReflektTransformer
+from reflekt.utils import segment_2_snake
 
 
 @click.group()
@@ -142,8 +148,6 @@ def init(project_dir_str: str) -> None:
         #     pass
         # elif plan_type == "snowplow":
         #     pass
-        # elif plan_type == "iteratively":
-        #     pass
 
         cdp_num_prompt = click.prompt(
             f"How do you collect first-party data?"
@@ -305,7 +309,7 @@ def new(plan_name: str) -> None:
     with open(plan_yml_file, "r") as f:
         doc = yaml.safe_load(f)
 
-    doc["display_name"] = plan_name
+    doc["name"] = plan_name
 
     with open(plan_yml_file, "w") as f:
         yaml.dump(doc, f)
@@ -356,8 +360,6 @@ def pull(plan_name: str, raw: bool, avo_branch: str) -> None:
         elif api.type.lower() == "segment":
             plan = SegmentPlan(plan_json)
         # TODO: Add support for other tracking plan types
-        # elif config.plan_type.lower() == "iteratively":
-        #     plan = IterativelyPlan(plan_json)
         # elif config.plan_type.lower() == "rudderstack":
         #     plan = RudderstackPlan(plan_json)
         # elif config.plan_type.lower() == "snowplow":
@@ -369,7 +371,6 @@ def pull(plan_name: str, raw: bool, avo_branch: str) -> None:
         logger.success(f"Built Reflekt tracking plan '{plan_name}'")
 
 
-@click.command()
 @click.option(
     "-n",
     "--name",
@@ -380,24 +381,178 @@ def pull(plan_name: str, raw: bool, avo_branch: str) -> None:
 @click.option(
     "--dry",
     is_flag=True,
+    required=False,
     help="Output JSON to be synced, without actually syncing it.",
 )
-def push(plan_name, dry) -> None:
+@click.option(
+    "-u",
+    "--update",
+    "updates",
+    type=str,
+    multiple=True,
+    required=False,
+    help=(
+        "Update/add specified (using kebab-case) event, all user traits, or all "
+        "group traits to a tracking plan. "
+    ),
+)
+@click.option(
+    "-r",
+    "--remove",
+    "removes",
+    type=str,
+    multiple=True,
+    required=False,
+    help=(
+        "Remove specified (using kebab-case) event, all user traits, or all group "
+        "traits from a tracking plan."
+    ),
+)
+@click.command()
+def push(plan_name, dry, updates, removes) -> None:
     """Sync tracking plan to CDP or Analytics Governance tool."""
     api = ReflektApiHandler().get_api()
-    if api.type.lower() in ["avo", "iteratively"]:
+    config = ReflektConfig()
+
+    if api.type.lower() in ["avo"]:
         logger.error(f"'reflekt push' not supported for {titleize(api.type)}.")
         raise click.Abort()
 
     plan_dir = ReflektProject().project_dir / "tracking-plans" / plan_name
-    logger.info(f"Loading Reflekt tracking plan '{plan_name}'")
-    loader = ReflektLoader(plan_dir=plan_dir, plan_name=plan_name)
-    reflekt_plan = loader.plan
-    transformer = ReflektTransformer(reflekt_plan)
-    cdp_plan = transformer.build_cdp_plan()
+
+    # Check for incompatible arguments
+    if updates != () and removes != ():
+        logger.error("--update and --remove arguments cannot be combined!")
+        raise SystemExit(1)
+
+    if updates != ():
+        # Determine what needs to be updated based on --update arg
+        update_events = tuple(
+            update for update in updates if update not in ("user-traits", "group-traits")
+        )
+        update_user_traits = tuple(
+            update for update in updates if update == "user-traits"
+        )
+        update_group_traits = tuple(
+            update for update in updates if update == "group-traits"
+        )
+        # Get the existing tracking plan that will be updated
+        logger.warning(
+            f"--update flag detected. Searching {titleize(config.plan_type)} for "
+            f"existing tracking plan '{plan_name}'"
+        )
+        existing_plan_json = api.get(plan_name)  # Get the existing plan
+        logger.info("")  # Terminal newline
+        logger.info(f"Found existing tracking plan '{plan_name}'")
+
+        # Load a Reflekt tracking, only including items from the --update arg
+        logger.info("")  # Terminal newline
+        logger.info(f"Updating tracking plan {plan_name}")
+        loader = ReflektLoader(
+            plan_dir=plan_dir,
+            parse_all=False,
+            events=update_events,
+            user_traits=update_user_traits,
+            group_traits=update_group_traits,
+        )
+        reflekt_plan = loader.plan
+        transformer = ReflektTransformer(reflekt_plan)
+        plan_updates_json = transformer.build_cdp_plan()
+        sync_events = []  # Events to sync
+
+        # For existing events - check if they need to be updated
+        for existing_event in existing_plan_json["rules"]["events"]:
+            match_found = False
+            match_event = None
+            for updated_event in plan_updates_json["tracking_plan"]["rules"]["events"]:
+                # If updated event name+version matches existing event, use updated event
+                if (
+                    updated_event["name"] == existing_event["name"]
+                    and updated_event["version"] == existing_event["version"]
+                ):
+                    match_found = True
+                    match_event = updated_event
+
+            if match_found:  # If match found, add updated event to sync list
+                sync_events.append(match_event)
+            else:  # If no match found, add existing event to sync list
+                sync_events.append(existing_event)
+
+        # For proposed events - check if they need to be added (not in existing events)
+        for updated_event in plan_updates_json["tracking_plan"]["rules"]["events"]:
+            match_found = False
+            for existing_event in existing_plan_json["rules"]["events"]:
+                # Check if updated event is already in existing events
+                if (
+                    updated_event["name"] == existing_event["name"]
+                    and updated_event["version"] == existing_event["version"]
+                ):
+                    match_found = True
+
+            if not match_found:  # Only ADD updated event if NOT found in existing events
+                sync_events.append(updated_event)
+
+        # Set the events to sync
+        sync_plan_json = copy.deepcopy(plan_updates_json)
+        sync_plan_json["tracking_plan"]["rules"]["events"] = sync_events
+        cdp_plan = sync_plan_json
+
+    elif removes != ():
+        # Determine what needs to be removed based on --remvoe arg
+        remove_events = tuple(
+            remove
+            for remove in removes
+            if remove != "user-traits" or remove != "group-traits"
+        )
+        remove_user_traits = tuple(
+            remove for remove in removes if remove == "user-traits"
+        )
+        remove_group_traits = tuple(
+            remove for remove in removes if remove == "group-traits"
+        )
+        # Get the existing plan that will have events/traits removed
+        remove_event_str = ", ".join("'" + event + "'" for event in remove_events)
+        logger.warning(
+            f"--remove flag detected. Removing event(s): {remove_event_str}, from "
+            f"tracking plan {plan_name} in {titleize(config.plan_type)}"
+        )
+        existing_plan_json = api.get(plan_name)
+        existing_events = existing_plan_json["rules"]["events"]
+        sync_events = []  # Events to sync
+
+        for existing_event in existing_events:
+            if dasherize(segment_2_snake(existing_event["name"])) not in remove_events:
+                sync_events.append(existing_event)
+
+        cdp_plan = copy.deepcopy(segment_payload_schema)
+        cdp_plan["tracking_plan"].update(segment_plan_schema)
+        cdp_plan["tracking_plan"]["rules"]["events"] = sync_events
+
+        if remove_user_traits != ():
+            cdp_plan["tracking_plan"]["rules"]["identify"]["properties"]["traits"][
+                "properties"
+            ] = []
+
+        if remove_group_traits != ():
+            cdp_plan["tracking_plan"]["rules"]["group"]["properties"]["traits"][
+                "properties"
+            ] = []
+
+    else:
+        logger.info(f"Loading Reflekt tracking plan '{plan_name}'")
+
+        # Load the plan using the ORIGINAL name
+        loader = ReflektLoader(
+            plan_dir=plan_dir,
+        )
+        reflekt_plan = loader.plan
+        transformer = ReflektTransformer(reflekt_plan)
+        plan_updates_json = transformer.build_cdp_plan()
+        cdp_plan = plan_updates_json
 
     if dry:
         payload = api.sync(plan_name, cdp_plan, dry=True)
+        logger.info("")  # Terminal newline
         logger.info(
             f"[DRY RUN] The following JSON would be sent to {transformer.plan_type}"
         )
@@ -406,25 +561,25 @@ def push(plan_name, dry) -> None:
         logger.info("")  # Terminal newline
         logger.info(
             f"Syncing converted tracking plan '{plan_name}' to "
-            f"{titleize(transformer.plan_type)}"
+            f"{titleize(config.plan_type)}"
         )
+
         api.sync(plan_name, cdp_plan)
         logger.info("")  # Terminal newline
         logger.success(
             f"Synced Reflekt tracking plan '{plan_name}' to "
-            f"{titleize(transformer.plan_type)}"
+            f"{titleize(config.plan_type)}"
         )
 
 
-@click.command()
 @click.option(
-    "-e",
-    "--event",
-    "events",
+    "-s",
+    "--select",
+    "selection",
     type=str,
     multiple=True,
     required=False,
-    help="Name of a single event to be tested (in kebab-case).",
+    help="Selected event or traits to be tested (specified in kebab-case).",
 )
 @click.option(
     "-n",
@@ -432,19 +587,41 @@ def push(plan_name, dry) -> None:
     "plan_name",
     type=str,
     required=True,
-    help="Name of tracking plan to be tested (in kebab-case).",
+    help="Name of tracking plan to be tested (specified in kebab-case).",
 )
-def test(plan_name, events) -> None:
+@click.command()
+def test(plan_name, selection) -> None:
     """Test tracking plan schema for naming, data types, and metadata."""
     plan_dir = ReflektProject().project_dir / "tracking-plans" / plan_name
     logger.info(f"Testing Reflekt tracking plan '{plan_name}'")
 
-    # Initialize ReflektLoader() always runs checks. Simple, not elegant.
-    ReflektLoader(plan_dir=plan_dir, plan_name=plan_name, events=events)
+    if selection != ():
+        # Determine what needs to be updated based on --update arg
+        selected_events = tuple(
+            selected
+            for selected in selection
+            if selected not in ("user-traits", "group-traits")
+        )
+        selected_user_traits = tuple(
+            selected for selected in selection if selected == "user-traits"
+        )
+        selected_group_traits = tuple(
+            selected for selected in selection if selected == "group-traits"
+        )
 
+        # Initialize ReflektLoader() always runs checks. Simple, not elegant.
+        ReflektLoader(
+            plan_dir=plan_dir,
+            parse_all=False,
+            events=selected_events,
+            user_traits=selected_user_traits,
+            group_traits=selected_group_traits,
+        )
+    else:
+        ReflektLoader(plan_dir=plan_dir)
     # If no errors are thrown, passed tests
     logger.info("")
-    logger.success("Testing completed. No errors detected")
+    logger.success("Test complete. No errors.")
 
 
 @click.option(
@@ -524,8 +701,6 @@ def dbt(
     pkg_name = f"reflekt_{project_name}_{cdp}"
     project_dir / "dbt-packages" / pkg_name
     dbt_project_yml = project.project_dir / "dbt_project.yml"
-    # blank_pkg_template = pkg_resources.resource_filename("reflekt", "templates/dbt/")
-    # tmp_pkg_dir = project_dir / ".reflekt" / "tmp" / pkg_name
     config.warehouse_type
 
     # Determine dbt pkg version to pass to ReflektTransformer
@@ -592,23 +767,10 @@ def dbt(
             logger.info("")  # Terminal newline
             version = existing_version
 
-    # # TODO: move this logic. Setup temporary dbt pkg
-    # if dbt_pkg_dir.exists():
-    #     # If dbt pk exists, use it as template
-    #     if tmp_pkg_dir.exists():
-    #         shutil.rmtree(tmp_pkg_dir)
-    #     shutil.copytree(dbt_pkg_dir, str(tmp_pkg_dir))
-    # else:
-    #     if tmp_pkg_dir.exists():  # ensure tmp dir is empty
-    #         shutil.rmtree(tmp_pkg_dir)
-    #     # If dbt pkg doesn't exist, use blank template
-    #     shutil.copytree(blank_pkg_template, str(tmp_pkg_dir))
-
     # Load Reflekt plan from 'tracking-plans/'
     logger.info(f"Loading Reflekt tracking plan {plan_name}")
     loader = ReflektLoader(
         plan_dir=plan_dir,
-        plan_name=plan_name,
         schema_name=schema,
         events=events,
     )
@@ -719,31 +881,30 @@ cli.add_command(dbt)
 
 # Used for CLI debugging
 if __name__ == "__main__":
-    # Call CLI command here with arguments as a list
+
+    # ----- GENERAL -----
     # init(["--project-dir", "~/Repos/test-repo"])
-    # new(["--project-dir", "test-plan"])
-    # pull(["--name", "my-plan"])
-    # push(["--name", "my-plan"])
-    # test(
-    #     [
-    #         "--name",
-    #         "my-plan",
-    #         "-e",
-    #         "cart-viewed",
-    #         "-e",
-    #         "product-added",
-    #     ]
-    # )
-    dbt(
-        [
-            "--name",
-            "my-plan",
-            # "--schema",
-            # "my_app_web",
-            # "--force-version",
-            # "0.1.0",
-        ]
-    )
-    # pull(["--name", "tracking-plan-example"])
-    # push(["--name", "tracking-plan-example"])
-    # test(["--name", "tracking-plan-example"])
+    # new(["--project-dir", "my-new-plan"])
+
+    # ----- AVO -----
+    # pull(["-n", "my-avo-plan"])
+    # pull(["-n", "my-avo-plan"], "--raw")
+    # pull(["-n", "my-avo-plan", "--avo-branch", "staging"])
+
+    # ----- SEGMENT -----
+    pull(["-n", "my-segment-plan"])
+    # pull(["-n", "my-segment-plan"], "--raw")
+    # push(["-n", "my-segment-plan"])
+    # push(["-n", "my-segment-plan", "--dry"])
+    # push(["-n", "my-segment-plan", "-u", "new-event"])
+    # push(["-n", "my-segment-plan", "-r", "new-event"])
+    # push(["-n", "test-plan", "-u", "user-traits"])
+
+    # ----- REFLEKT -----
+    # test(["-n", "my-segment-plan"])
+    # test(["-n", "my-segment-plan", "-e", "order-completed"])
+
+    # dbt(["-n", "my-segment-plan"])
+    # dbt(["-n", "my-segment-plan", "-e", "order-completed")]
+    # dbt(["-n", "my-segment-plan", "-e", "user-traits"])
+    # dbt(["-n", "my-segment-plan", "-e", "group-traits"])
