@@ -14,7 +14,7 @@ from loguru import logger
 from packaging.version import Version
 
 from reflekt.config import ReflektConfig
-from reflekt.constants import REFLEKT_RESERVED_COLUMNS, REFLEKT_TEMPLATE_COLUMNS
+from reflekt.constants import REFLEKT_INJECTED_COLUMNS, REFLEKT_TEMPLATED_COLUMNS
 from reflekt.dbt import (
     dbt_column_schema,
     dbt_doc_schema,
@@ -28,7 +28,6 @@ from reflekt.plan import ReflektPlan
 from reflekt.project import ReflektProject
 from reflekt.property import ReflektProperty
 from reflekt.segment.columns import (
-    seg_event_cols,
     seg_groups_cols,
     seg_identify_cols,
     seg_pages_cols,
@@ -38,7 +37,6 @@ from reflekt.segment.columns import (
 )
 from reflekt.segment.schema import (
     segment_event_schema,
-    segment_items_schema,
     segment_payload_schema,
     segment_plan_schema,
     segment_property_schema,
@@ -52,9 +50,6 @@ class ReflektTransformer(object):
     def __init__(
         self,
         reflekt_plan: ReflektPlan,
-        # dbt_package_name: Optional[str] = None,
-        # dbt_pkg_dir: Optional[Path] = None,
-        # tmp_pkg_dir: Optional[Path] = None,
         models_subfolder: Optional[str] = None,
         dbt_package_version: Optional[Version] = None,
     ) -> None:
@@ -103,7 +98,6 @@ class ReflektTransformer(object):
             return self._build_plan_segment(self.reflekt_plan)
         elif plan_type.lower() == "snowplow":
             return self._build_plan_snowplow(self.reflekt_plan)
-        # 'reflekt push' not supported for Avo or Iteratively.
 
     def _build_plan_rudderstack(self):
         pass
@@ -126,8 +120,9 @@ class ReflektTransformer(object):
                 segment_event = self._build_segment_event(reflekt_event)
                 segment_plan["rules"]["events"].append(segment_event)
 
-        if reflekt_plan.identify_traits != []:
-            for reflekt_identify_trait in reflekt_plan.identify_traits:
+        if reflekt_plan.user_traits != []:
+            logger.info("    Converting user traits")
+            for reflekt_identify_trait in reflekt_plan.user_traits:
                 segment_identify_trait = self._build_segment_trait(
                     reflekt_identify_trait
                 )
@@ -142,6 +137,7 @@ class ReflektTransformer(object):
                 ].update({reflekt_identify_trait.name: segment_identify_trait})
 
         if reflekt_plan.group_traits != []:
+            logger.info("    Converting group traits")
             for reflekt_group_trait in reflekt_plan.group_traits:
                 segment_group_trait = self._build_segment_trait(reflekt_group_trait)
 
@@ -185,101 +181,91 @@ class ReflektTransformer(object):
 
         return segment_event
 
-    def _build_segment_property(self, reflekt_property: ReflektProperty) -> dict:
+    def _build_segment_property(
+        self, reflekt_property: Union[ReflektProperty, ReflektTrait]
+    ) -> dict:
+        # If self._build_segment_property() is called recursively to build array or
+        # object properties, need to convert the array_item_schema or object_properties
+        # attribute, which is a dict, to a ReflektProperty
+        if isinstance(reflekt_property, dict):
+            reflekt_property = ReflektProperty(reflekt_property)
+
+        # For each Reflekt property, create a fresh segment property object
         segment_property = copy.deepcopy(segment_property_schema)
+        # Update the segment property description and name
         segment_property["description"] = reflekt_property.description
         segment_property["type"] = [reflekt_property.type]
-        segment_property = self._parse_reflekt_property(
-            reflekt_property, segment_property
-        )
 
-        return segment_property
-
-    def _parse_reflekt_property(
-        self,
-        reflekt_property: Union[ReflektProperty, ReflektTrait],
-        segment_property: dict,
-    ) -> dict:
-        # NOTE - ReflektTrait is a child class of ReflektProperty, so this
-        # instance method will parse Reflekt properties and traits
-        updated_segment_property = copy.deepcopy(segment_property)
-        if hasattr(reflekt_property, "allow_null") and reflekt_property.allow_null:
-            updated_segment_property["type"].append("null")
-        elif hasattr(reflekt_property, "type") and reflekt_property.type == "any":
+        # Check for property configurations and set accordingly
+        if hasattr(reflekt_property, "type") and reflekt_property.type == "any":
             # If data type is 'any', delete type key. Otherwise, triggers
             # 'invalid argument' from Segment API
-            updated_segment_property.pop("type", None)
+            segment_property.pop("type", None)
+        elif hasattr(reflekt_property, "allow_null") and reflekt_property.allow_null:
+            segment_property["type"].append("null")
         elif (
             hasattr(reflekt_property, "pattern") and reflekt_property.pattern is not None
         ):
-            updated_segment_property["pattern"] = reflekt_property.pattern
+            segment_property["pattern"] = reflekt_property.pattern
         elif hasattr(reflekt_property, "enum") and reflekt_property.enum is not None:
-            updated_segment_property["enum"] = reflekt_property.enum
+            segment_property["enum"] = reflekt_property.enum
         elif hasattr(reflekt_property, "datetime") and reflekt_property.datetime:
-            updated_segment_property["format"] = "date-time"
-
-        if (
+            segment_property["format"] = "date-time"
+        elif (
             hasattr(reflekt_property, "array_item_schema")
             and reflekt_property.array_item_schema is not None
         ):
-            segment_array_items = copy.deepcopy(segment_items_schema)
-            for reflekt_item_property in reflekt_property.array_item_schema:
-                segment_item_property = copy.deepcopy(segment_property_schema)
-                segment_item_property["description"] = reflekt_item_property[
-                    "description"
-                ]
-                segment_item_property["type"] = [reflekt_item_property["type"]]
-                segment_item_property = self._parse_reflekt_property(
-                    reflekt_item_property, segment_item_property
+            # segment_array_items = copy.deepcopy(segment_items_schema)
+            segment_property["items"] = {
+                "type": "object",
+                "properties": {},
+                "required": [],
+            }
+            for reflekt_array_item_property in reflekt_property.array_item_schema:
+                segment_item_property = self._build_segment_property(  # Recursion!
+                    reflekt_array_item_property
+                )
+                segment_property["items"]["properties"].update(
+                    {reflekt_array_item_property["name"]: segment_item_property}
                 )
 
                 if (
-                    "required" in reflekt_item_property
-                    and reflekt_item_property["required"]
+                    "required" in reflekt_array_item_property
+                    and reflekt_array_item_property["required"]
                 ):
-                    segment_array_items["items"]["required"].append(
-                        reflekt_item_property["name"]
+                    segment_property["items"]["required"].append(
+                        reflekt_array_item_property["name"]
                     )
-
-                segment_array_items["items"]["properties"].update(
-                    {reflekt_item_property["name"]: segment_item_property}
-                )
-
-            updated_segment_property.update(segment_array_items)
-
-        if (
+        elif (
             hasattr(reflekt_property, "object_properties")
             and reflekt_property.object_properties is not None
         ):
-            updated_segment_property["properties"] = {}
-            updated_segment_property["required"] = []
+            segment_property["properties"] = {}
+            segment_property["required"] = []
 
-            for object_property in reflekt_property.object_properties:
-                segment_property = copy.deepcopy(segment_property_schema)
-                segment_property["description"] = object_property["description"]
-                segment_property["type"] = object_property["type"]
-                updated_segment_property["properties"].update(
-                    {object_property["name"]: segment_property}
+            for reflekt_object_property in reflekt_property.object_properties:
+                segment_object_property = self._build_segment_property(  # Recursion!
+                    reflekt_object_property
+                )
+                segment_property["properties"].update(
+                    {reflekt_object_property["name"]: segment_object_property}
                 )
 
-                if "required" in object_property and object_property["required"]:
-                    updated_segment_property["required"].append(object_property["name"])
+                if (
+                    "required" in reflekt_object_property
+                    and reflekt_object_property["required"]
+                ):
+                    segment_property["required"].append(reflekt_object_property["name"])
 
-        return updated_segment_property
+        return segment_property
 
     def _build_segment_trait(self, reflekt_trait: ReflektTrait) -> dict:
-        segment_trait = copy.deepcopy(segment_property_schema)
-        segment_trait["description"] = reflekt_trait.description
-        segment_trait["type"] = [reflekt_trait.type]
-        # _parse_reflekt_property works on traits too
-        segment_trait = self._parse_reflekt_property(reflekt_trait, segment_trait)
-
-        return segment_trait
-
-    def check_for_schema_match(self) -> None:
-        pass
+        # Although conceptually different, ReflektProperty and ReflektTrait are
+        # structurally the same. Therefore, we can reuse self._build_segment_property()
+        return self._build_segment_property(reflekt_trait)
 
     def build_dbt_package(self) -> None:
+        self.db_errors = []
         logger.info(
             f"Building Reflekt dbt package:"
             f"\n        dbt package name: {self.dbt_package_name}"
@@ -332,124 +318,145 @@ class ReflektTransformer(object):
             models_subfolder_dir.mkdir(parents=True, exist_ok=True)
 
         if self.cdp_name == "rudderstack":
-            return self._dbt_package_rudderstack(
-                self.reflekt_plan, self.models_subfolder
-            )
+            return self._dbt_package_rudderstack(self.reflekt_plan)
         elif self.cdp_name == "segment":
-            return self._dbt_package_segment(self.reflekt_plan, self.models_subfolder)
+            return self._dbt_package_segment(self.reflekt_plan)
         elif self.cdp_name == "snowplow":
-            return self._dbt_package_snowplow(self.reflekt_plan, self.models_subfolder)
+            return self._dbt_package_snowplow(self.reflekt_plan)
 
     def _dbt_package_rudderstack(
         self,
         reflekt_plan: ReflektPlan,
-        models_subfolder: str,
     ) -> None:
         pass
 
     def _dbt_package_snowplow(
         self,
         reflekt_plan: ReflektPlan,
-        models_subfolder: str,
     ) -> None:
         pass
 
     def _dbt_package_segment(
         self,
         reflekt_plan: ReflektPlan,
-        models_subfolder: str,
     ) -> None:
-        self.db_errors = []
-
-        # Setup `Page Viewed` and `Screen Viewed` events, if they exist in Reflekt plan
-        page_viewed_event = [
-            event
-            for event in reflekt_plan.events
-            if str.lower(event.name).replace("_", " ").replace("-", " ") == "page viewed"
-        ]
-
-        if page_viewed_event == []:  # Page Viewed not in tracking plan
-            page_viewed_props = []
-        else:
-            page_viewed_props = page_viewed_event[0].properties
-
-        screen_viewed_event = [
-            event
-            for event in reflekt_plan.events
-            if str.lower(event.name).replace("_", " ").replace("-", " ")
-            == "screen viewed"
-        ]
-
-        if screen_viewed_event == []:  # Screen Viewed not in tracking plan
-            screen_viewed_props = []
-        else:
-            screen_viewed_props = screen_viewed_event[0].properties
-
-        user_traits = [trait for trait in reflekt_plan.identify_traits]
-        group_traits = [trait for trait in reflekt_plan.group_traits]
-
-        std_segment_tables = [
-            {
-                "name": "identifies",
-                "description": "A table with identify() calls.",  # noqa: E501
-                "unique_key": "identify_id",
-                "cdp_cols": seg_identify_cols,
-                "plan_cols": user_traits,
-            },
-            {
-                "name": "users",
-                "description": "A table with the latest traits for users.",  # noqa: E501
-                "unique_key": "user_id",
-                "cdp_cols": seg_users_cols,
-                "plan_cols": user_traits,
-            },
-            {
-                "name": "groups",
-                "description": "A table with group() calls.",  # noqa: E501
-                "unique_key": "group_id",
-                "cdp_cols": seg_groups_cols,
-                "plan_cols": group_traits,
-            },
-            {
-                "name": "pages",
-                "description": "A table with page() calls.",  # noqa: E501
-                "unique_key": "page_id",
-                "cdp_cols": seg_pages_cols,
-                "plan_cols": page_viewed_props,
-            },
-            {
-                "name": "screens",
-                "description": "A table with screen() calls.",  # noqa: E501
-                "unique_key": "screen_id",
-                "cdp_cols": seg_screens_cols,
-                "plan_cols": screen_viewed_props,
-            },
-            {
-                "name": "tracks",
-                "description": "A table with track() event calls.",  # noqa: E501
-                "unique_key": "event_id",
-                "cdp_cols": seg_tracks_cols,
-                "plan_cols": [],
-            },
-        ]
-
         dbt_src = self._template_dbt_source(reflekt_plan=reflekt_plan)
 
+        std_segment_tables = []
+        if self.reflekt_plan.user_traits:
+            std_segment_tables.append(
+                {
+                    "name": "identifies",
+                    "description": "A table with identify() calls.",
+                    "unique_key": "identify_id",
+                    "cdp_cols": seg_identify_cols,
+                    "plan_cols": reflekt_plan.user_traits,
+                }
+            )
+            std_segment_tables.append(
+                {
+                    "name": "users",
+                    "description": "A table with the latest traits for users.",
+                    "unique_key": "user_id",
+                    "cdp_cols": seg_users_cols,
+                    "plan_cols": reflekt_plan.user_traits,
+                }
+            )
+
+        if self.reflekt_plan.group_traits:
+            std_segment_tables.append(
+                {
+                    "name": "groups",
+                    "description": "A table with group() calls.",
+                    "unique_key": "group_id",
+                    "cdp_cols": seg_groups_cols,
+                    "plan_cols": reflekt_plan.group_traits,
+                }
+            )
+
+        if self.reflekt_plan.events:
+            std_segment_tables.append(
+                {
+                    "name": "tracks",
+                    "description": "A table with event track() calls.",
+                    "unique_key": "event_id",
+                    "cdp_cols": seg_tracks_cols,
+                    "plan_cols": [],
+                }
+            )
+
         for std_segment_table in std_segment_tables:
-            if self.plan_type == "avo" and std_segment_table["name"] in [
-                "identifies",
-                "users",
-                "groups",
-            ]:
-                pass
+            db_columns, error_msg = self.db_engine.get_columns(
+                self.schema, std_segment_table["name"]
+            )
+            if error_msg is not None:
+                self.db_errors.append(error_msg)
             else:
+                table_name = std_segment_table["name"]
+                model_name = (
+                    f"{self.model_prefix}{self.schema}__{table_name}"
+                    if self.schema_alias is None
+                    else f"{self.model_prefix}{self.schema_alias}__{table_name}"
+                )
+                doc_name = (
+                    f"{self.docs_prefix}{self.schema}__{table_name}"
+                    if self.schema_alias is None
+                    else f"{self.docs_prefix}{self.schema_alias}__{table_name}"
+                )
+                self._template_dbt_table(
+                    dbt_src=dbt_src,
+                    table_name=table_name,
+                    table_description=std_segment_table["description"],
+                )
+                self._template_dbt_model(
+                    models_subfolder=self.models_subfolder,
+                    materialized=self.materialized,
+                    unique_key=std_segment_table["unique_key"],
+                    table_name=table_name,
+                    model_name=model_name,
+                    db_columns=db_columns,
+                    cdp_cols=std_segment_table["cdp_cols"],
+                    plan_cols=std_segment_table["plan_cols"],
+                )
+                self._template_dbt_doc(
+                    models_subfolder=self.models_subfolder,
+                    doc_name=doc_name,
+                    model_name=model_name,
+                    model_description=std_segment_table["description"],
+                    db_columns=db_columns,
+                    cdp_cols=std_segment_table["cdp_cols"],
+                    plan_cols=std_segment_table["plan_cols"],
+                )
+
+        for event in self.reflekt_plan.events:
+            event_name_lower = str.lower(event.name).replace("_", " ").replace("-", " ")
+            if event_name_lower == "page viewed":
+                table_name = "pages"
+                unique_key = "page_id"
+                cdp_cols = seg_pages_cols
+            elif event_name_lower == "screen viewed":
+                table_name = "screens"
+                unique_key = "screen_id"
+                cdp_cols = seg_screens_cols
+            else:
+                table_name = segment_2_snake(event.name)
+                unique_key = "event_id"
+                cdp_cols = seg_tracks_cols
+
+            # Get event versions
+            multi_version_event_list = [
+                x for x in reflekt_plan.events if x.name == event.name
+            ]
+            event_versions = [event.version for event in multi_version_event_list]
+
+            # Only template latest version of event
+            if event.version == max(event_versions):
                 db_columns, error_msg = self.db_engine.get_columns(
-                    self.schema, std_segment_table["name"]
+                    self.schema, table_name
                 )
                 if error_msg is not None:
                     self.db_errors.append(error_msg)
                 else:
-                    table_name = std_segment_table["name"]
                     model_name = (
                         f"{self.model_prefix}{self.schema}__{table_name}"
                         if self.schema_alias is None
@@ -463,91 +470,34 @@ class ReflektTransformer(object):
                     self._template_dbt_table(
                         dbt_src=dbt_src,
                         table_name=table_name,
-                        table_description=std_segment_table["description"],
+                        table_description=event.description,
                     )
                     self._template_dbt_model(
                         models_subfolder=self.models_subfolder,
                         materialized=self.materialized,
-                        unique_key=std_segment_table["unique_key"],
+                        unique_key=unique_key,
                         table_name=table_name,
                         model_name=model_name,
                         db_columns=db_columns,
-                        cdp_cols=std_segment_table["cdp_cols"],
-                        plan_cols=std_segment_table["plan_cols"],
+                        cdp_cols=cdp_cols,
+                        plan_cols=event.properties,
+                        event_name=event.name,
                     )
                     self._template_dbt_doc(
                         models_subfolder=self.models_subfolder,
                         doc_name=doc_name,
                         model_name=model_name,
-                        model_description=std_segment_table["description"],
+                        model_description=event.description,
                         db_columns=db_columns,
-                        cdp_cols=std_segment_table["cdp_cols"],
-                        plan_cols=std_segment_table["plan_cols"],
+                        cdp_cols=cdp_cols,
+                        plan_cols=event.properties,
                     )
 
-        for event in reflekt_plan.events:
-            if event.name.lower() in ["page viewed", "screen viewed"]:
-                pass  # Already handled above by std_segment_tables for loop
-            else:
-                # Get event versions
-                multi_version_event_list = [
-                    x for x in reflekt_plan.events if x.name == event.name
-                ]
-                event_versions = [event.version for event in multi_version_event_list]
-
-                # Only template latest version of event
-                if event.version == max(event_versions):
-                    db_columns, error_msg = self.db_engine.get_columns(
-                        self.schema, segment_2_snake(event.name)
-                    )
-                    if error_msg is not None:
-                        self.db_errors.append(error_msg)
-                    else:
-                        table_name = segment_2_snake(event.name)
-                        model_name = (
-                            f"{self.model_prefix}{self.schema}__{table_name}"
-                            if self.schema_alias is None
-                            else f"{self.model_prefix}{self.schema_alias}__{table_name}"
-                        )
-                        doc_name = (
-                            f"{self.docs_prefix}{self.schema}__{table_name}"
-                            if self.schema_alias is None
-                            else f"{self.docs_prefix}{self.schema_alias}__{table_name}"
-                        )
-                        self._template_dbt_table(
-                            dbt_src=dbt_src,
-                            table_name=table_name,
-                            table_description=event.description,
-                        )
-                        self._template_dbt_model(
-                            models_subfolder=self.models_subfolder,
-                            materialized=self.materialized,
-                            unique_key="event_id",
-                            table_name=table_name,
-                            model_name=model_name,
-                            db_columns=db_columns,
-                            cdp_cols=seg_event_cols,
-                            plan_cols=event.properties,
-                            event_name=event.name,
-                        )
-                        self._template_dbt_doc(
-                            models_subfolder=self.models_subfolder,
-                            doc_name=doc_name,
-                            model_name=model_name,
-                            model_description=event.description,
-                            db_columns=db_columns,
-                            cdp_cols=seg_event_cols,
-                            plan_cols=event.properties,
-                        )
-
         src_name = (
             f"{self.src_prefix}{self.schema}"
             if self.schema_alias is None
             else f"{self.src_prefix}{self.schema_alias}"
         )
-        dbt_src_path = (
-            self.tmp_pkg_dir / "models" / self.models_subfolder / f"{src_name}.yml"
-        )
 
         if self.docs_in_folder:
             docs_folder_str = (
@@ -610,17 +560,40 @@ class ReflektTransformer(object):
         reflekt_plan: ReflektPlan,
     ) -> dict:
         logger.info(f"Initializing template for dbt source {self.schema}")
-        dbt_src = copy.deepcopy(dbt_src_schema)
-        dbt_src["sources"][0]["name"] = (
-            self.schema_alias if self.schema_alias is not None else self.schema
-        )
-        dbt_src["sources"][0]["schema"] = self.schema
-        dbt_src["sources"][0]["database"] = self.database
-        dbt_src["sources"][0]["description"] = (
-            f"Schema in {titleize(self.warehouse_type)} where data for the "
-            f"{reflekt_plan.name} {titleize(self.cdp_name)} source is stored."
+        src_name = (
+            f"{self.src_prefix}{self.schema}"
+            if self.schema_alias is None
+            else f"{self.src_prefix}{self.schema_alias}"
         )
 
+        if self.docs_in_folder:
+            docs_folder_str = (
+                str(self.tmp_pkg_dir) + f"/models/{self.models_subfolder}/docs"
+            )
+            docs_folder_path = Path(docs_folder_str)
+            docs_folder_path.mkdir(exist_ok=True)
+            dbt_src_path = docs_folder_path / f"{src_name}.yml"
+        else:
+            dbt_src_path = (
+                self.tmp_pkg_dir / "models" / self.models_subfolder / f"{src_name}.yml"
+            )
+
+        if dbt_src_path.exists():
+            with open(dbt_src_path, "r") as f:
+                dbt_src = yaml.safe_load(f)
+
+        else:
+            dbt_src = copy.deepcopy(dbt_src_schema)
+            dbt_src["sources"][0]["name"] = (
+                self.schema_alias if self.schema_alias is not None else self.schema
+            )
+            dbt_src["sources"][0]["schema"] = self.schema
+            dbt_src["sources"][0]["database"] = self.database
+            dbt_src["sources"][0]["description"] = (
+                f"Schema in {titleize(self.warehouse_type)} where raw event data for "
+                f"the {reflekt_plan.name} tracking plan is loaded."
+            )
+
         return dbt_src
 
     def _template_dbt_table(
@@ -629,9 +602,21 @@ class ReflektTransformer(object):
         table_name: str,
         table_description: str,
     ) -> None:
-        # Check that table does not already exist in dbt source. This can happen
-        # for Segment events with multiple versions
-        if table_name not in dbt_src["sources"][0]["tables"]:
+        # Get list of tables already in dbt source (if any)
+        table_names = [dbt_tbl["name"] for dbt_tbl in dbt_src["sources"][0]["tables"]]
+
+        # Table already defined in dbt source
+        if table_name in table_names:
+            logger.info("")  # Terminal newline
+            logger.info(f"Templating table '{table_name}' in dbt source {self.schema}")
+
+            for dbt_tbl in dbt_src["sources"][0]["tables"]:
+                # Find existing table entry with matching name, update name & description
+                if table_name == dbt_tbl["name"]:
+                    dbt_tbl["name"] = table_name
+                    dbt_tbl["description"] = table_description
+
+        else:
             logger.info("")  # Terminal newline
             logger.info(f"Templating table '{table_name}' in dbt source {self.schema}")
             dbt_tbl = copy.deepcopy(dbt_table_schema)
@@ -658,6 +643,7 @@ class ReflektTransformer(object):
         )
         logger.info("    Adding {{ config(...) }} to model SQL")
         mdl_sql = self._template_dbt_model_config(
+            table_name,
             materialized,
             unique_key,
             self.warehouse_type,
@@ -676,11 +662,11 @@ class ReflektTransformer(object):
         mdl_sql += "renamed as (\n\n" "    select"
 
         for column, mapped_columns in cdp_cols.items():
-            if column in db_columns or column in REFLEKT_TEMPLATE_COLUMNS:
+            if column in db_columns or column in REFLEKT_INJECTED_COLUMNS:
                 for mapped_column in mapped_columns:
                     if (
                         mapped_column["source_name"] in db_columns
-                        or column in REFLEKT_TEMPLATE_COLUMNS
+                        or column in REFLEKT_INJECTED_COLUMNS
                     ) and mapped_column["schema_name"] is not None:
                         logger.info(
                             f"    Adding column '{mapped_column['schema_name']}' to "
@@ -699,19 +685,23 @@ class ReflektTransformer(object):
                         mdl_sql += "\n        " + col_sql + ","
 
         for column in plan_cols:
-            if (
-                segment_2_snake(column.name) in db_columns
-                and segment_2_snake(column.name) not in REFLEKT_RESERVED_COLUMNS
-            ):
-                # If the columns is named 'interval', surround in double quotes
+            if segment_2_snake(column.name) in db_columns:
                 column_name = (
                     segment_2_snake(column.name)
-                    if column.name != "interval"  # Handle columns named "interval"
+                    if column.name != "interval"  # Handle column named "interval"
                     else '"interval"'
                 )
-                logger.info(f"    Adding column '{column_name}' to model SQL")
-                col_sql = column_name
-                mdl_sql += "\n        " + col_sql + ","
+
+                if segment_2_snake(column.name) in REFLEKT_TEMPLATED_COLUMNS:
+                    logger.warning(
+                        f"Column {column_name} collides with reserved Reflekt column "
+                        "with the same name. Prefixing with underscore."
+                    )
+                    logger.info(f"    Adding column '_{column_name}' to model SQL")
+                    mdl_sql += f"\n        {column_name} as _{column_name},"
+                else:
+                    logger.info(f"    Adding column '{column_name}' to model SQL")
+                    mdl_sql += f"\n        {column_name},"
 
         mdl_sql = mdl_sql[:-1]  # Remove final trailing comma
         # fmt: off
@@ -729,41 +719,47 @@ class ReflektTransformer(object):
 
     def _template_dbt_model_config(
         self,
+        table_name: str,
         materialized: str,
         unique_key: str,
         warehouse_type: str,
     ) -> str:
-        if materialized == "view":
-            # fmt: off
+        if table_name in ["users", "groups"]:
             model_config = (
-                "{{\n"
-                "  config(\n"
-                "    materialized = 'view',\n"
-                "  )\n"
-                "}}\n\n"
-            )
-            # fmt: on
-        elif materialized == "incremental":
-            if warehouse_type == "redshift":
-                sort_str = "sort = 'tstamp'"
-            elif warehouse_type == "snowflake":
-                sort_str = "cluster_by = 'tstamp'"
-            model_config = (
-                "{{\n"
-                f"  config(\n"
-                f"    materialized = 'incremental',\n"
-                f"    unique_key = '{unique_key}',\n"
-                f"    {sort_str}\n"
-                f"  )\n"
-                "}}\n\n"
+                "{{\n" "  config(\n" "    materialized = 'view',\n" "  )\n" "}}\n\n"
             )
         else:
-            logger.error(
-                "Invalid 'materialized:' config in reflekt_project.yml. Must be either "
-                "'view' or 'incremental'. See the Reflekt docs "
-                "(https://bit.ly/reflekt-project-config) on project configuration."
-            )
-            raise SystemExit(1)
+            if materialized == "view":
+                # fmt: off
+                model_config = (
+                    "{{\n"
+                    "  config(\n"
+                    "    materialized = 'view',\n"
+                    "  )\n"
+                    "}}\n\n"
+                )
+                # fmt: on
+            elif materialized == "incremental":
+                if warehouse_type == "redshift":
+                    sort_str = "sort = 'tstamp'"
+                elif warehouse_type == "snowflake":
+                    sort_str = "cluster_by = 'tstamp'"
+                model_config = (
+                    "{{\n"
+                    f"  config(\n"
+                    f"    materialized = 'incremental',\n"
+                    f"    unique_key = '{unique_key}',\n"
+                    f"    {sort_str}\n"
+                    f"  )\n"
+                    "}}\n\n"
+                )
+            else:
+                logger.error(
+                    "Invalid 'materialized:' config in reflekt_project.yml. Must be "
+                    "either 'view' or 'incremental'. See the Reflekt docs "
+                    "(https://bit.ly/reflekt-project-config) on project configuration."
+                )
+                raise SystemExit(1)
 
         return model_config
 
@@ -814,7 +810,7 @@ class ReflektTransformer(object):
         dbt_mdl_doc["description"] = model_description
 
         for column, mapped_columns in cdp_cols.items():
-            if column in db_columns or column in REFLEKT_TEMPLATE_COLUMNS:
+            if column in db_columns or column in REFLEKT_INJECTED_COLUMNS:
                 for mapped_column in mapped_columns:
                     if mapped_column["schema_name"] is not None:
                         logger.info(
@@ -840,15 +836,25 @@ class ReflektTransformer(object):
                         dbt_mdl_doc["columns"].append(mdl_col)
 
         for column in plan_cols:
-            if (
-                segment_2_snake(column.name) in db_columns
-                and segment_2_snake(column.name) not in REFLEKT_RESERVED_COLUMNS
-            ):
-                logger.info(
-                    f"    Adding column '{segment_2_snake(column.name)}' to docs"
-                )
+            if segment_2_snake(column.name) in db_columns:
                 mdl_col = copy.deepcopy(dbt_column_schema)
-                mdl_col["name"] = segment_2_snake(column.name)
+
+                if segment_2_snake(column.name) in REFLEKT_TEMPLATED_COLUMNS:
+                    logger.warning(
+                        f"Column {segment_2_snake(column.name)} collides with reserved "
+                        f"Reflekt column with the same name. Prefixing with with "
+                        f"underscore."
+                    )
+                    logger.info(
+                        f"    Adding column '_{segment_2_snake(column.name)}' to docs"
+                    )
+                    mdl_col["name"] = f"_{segment_2_snake(column.name)}"
+                else:
+                    logger.info(
+                        f"    Adding column '{segment_2_snake(column.name)}' to docs"
+                    )
+                    mdl_col["name"] = f"{segment_2_snake(column.name)}"
+
                 mdl_col["description"] = column.description
                 dbt_mdl_doc["columns"].append(mdl_col)
 
