@@ -5,11 +5,11 @@
 from typing import Optional, Tuple
 
 import sqlalchemy
+from loguru import logger
 from snowflake.sqlalchemy import URL as snow_url
 from sqlalchemy.engine.url import URL as redshift_url
 
 from reflekt.config import ReflektConfig
-from reflekt.errors import ReflektConfigError
 
 
 class WarehouseConnection:
@@ -49,11 +49,13 @@ class WarehouseConnection:
             )
 
         else:
-            raise ReflektConfigError(
-                f"Invalid warehouse type specified in {self._config.path}. See "
-                f"Reflekt docs on Reflekt Profile configuration:"
-                f"    https://reflekt-ci.notion.site/Reflekt-Profile-Configuration-8ee71745477b4c91a6aafbb025cf2280"  # noqa: E501
+            logger.error(
+                f"Invalid warehouse type specified in {self._config.path}. See the "
+                f"Reflekt profile configuration docs "
+                f"(https://bit.ly/reflekt-profile-config) for details on profile "
+                f"configuration."
             )
+            raise SystemExit(1)
 
     def get_columns(
         self, schema: str, table_name: str
